@@ -2,14 +2,15 @@ from test_utils import *
 from minpower.results import make_solution
 
 @istest
+@with_setup(teardown=reset_config)
 def check_power_status():
     '''
     Ensure that the solution generator_status and generator_power attributes
     match with the optimization values returned.
     '''
     generators=[
-        make_cheap_gen(Pmax=100),
-        make_mid_gen(Pmax=20),
+        make_cheap_gen(pmax=100),
+        make_mid_gen(pmax=20),
         make_expensive_gen()
     ]
 
